#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Run Django Test with Python setuptools test command


REFERENCE:
    http://gremu.net/blog/2010/enable-setuppy-test-your-django-apps/

AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
import os, sys
os.environ['DJANGO_SETTINGS_MODULE'] = 'miniblog.settings'
test_dir = os.path.join(os.path.dirname(__file__), 'tests', 'src')
sys.path.insert(0, test_dir)

from django.test.utils import get_runner
from django.conf import settings

def runtests(verbosity=1, interactive=True):
    """Run Django Test"""
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=verbosity, interactive=interactive)
    failures = test_runner.run_tests(['universaltag', 'blogs'])
    sys.exit(bool(failures))

if __name__ == '__main__':
    runtests()




#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Django App TestCase


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django import test
from django.conf import settings
from django.core.management import call_command
from django.db.models import loading
from django.contrib.contenttypes.models import ContentType

class AppTestCase(test.TestCase):
    installed_apps = []
    middleware_classes = []

    def _get_installed_apps(self):
        # ContentType is required so add
        self.installed_apps.append('django.contrib.contenttypes')
        _installed_apps = list(settings.INSTALLED_APPS)
        for app in self.installed_apps:
            if app not in _installed_apps:
                _installed_apps.append(app)
        return _installed_apps
    def _get_middleware_classes(self):
        _middleware_classes = list(settings.MIDDLEWARE_CLASSES)
        for middleware_class in self.middleware_classes:
            if middleware_class not in _middleware_classes:
                _middleware_classes.append(middleware_class)
        return _middleware_classes

    def _clear_all_meta_caches(self):
        def clear_meta_caches(meta):
            CACHE_NAMES = (
                '_m2m_cache',
                '_field_cache',
                '_name_map',
                '_related_objects_cache',
                '_related_many_to_many_cache',
            )
            for cache_name in CACHE_NAMES:
                if hasattr(meta, cache_name):
                    delattr(meta, cache_name)
        for ct in ContentType.objects.iterator():
            meta = ct.model_class()._meta
            clear_meta_caches(meta)

    def _pre_setup(self):
        # store original installed apps/middleware classes
        self._original_installed_apps = list(settings.INSTALLED_APPS)
        self._original_middleware_classes = list(settings.MIDDLEWARE_CLASSES)
        # get extra required apps and middleware
        settings.INSTALLED_APPS = self._get_installed_apps()
        settings.MIDDLEWARE_CLASSES = self._get_middleware_classes()
        # Call syncdb to create db for extra apps (migrate=False for South)
        loading.cache.loaded = False
        call_command('syncdb', interactive=False, verbosity=0, migrate=False)
        # Call the original method that does the fixtures etc.
        super(AppTestCase, self)._pre_setup()

        # Clear cache of meta of all models
        self._clear_all_meta_caches()

    def _post_teardown(self):
        # Call the original method
        super(AppTestCase, self)._post_teardown()
        # Restore the settings
        settings.INSTALLED_APPS = self._original_installed_apps
        settings.MIDDLEWARE_CLASSES = self._original_middleware_classes
        loading.cache.loaded = False



# -*- coding: utf-8 -*-
#
# Author:        alisue
# Date:            2010/10/26
#
# Copied from django-tagging.utils
#
from django.utils.encoding import force_unicode

def parse_tag_input(input):
    """
    Parses tag input, with multiple word input being activated and
    delineated by commas and double quotes. Quotes take precedence, so
    they may contain commas.

    Returns a sorted list of unique tag names.
    """
    if not input:
        return []

    input = force_unicode(input)

    # Special case - if there are no commas or double quotes in the
    # input, we don't *do* a recall... I mean, we know we only need to
    # split on spaces.
    if u',' not in input and u'"' not in input:
        words = list(set(split_strip(input, u' ')))
        words.sort()
        return words

    words = []
    buffer = []
    # Defer splitting of non-quoted sections until we know if there are
    # any unquoted commas.
    to_be_split = []
    saw_loose_comma = False
    open_quote = False
    i = iter(input)
    try:
        while 1:
            c = i.next()
            if c == u'"':
                if buffer:
                    to_be_split.append(u''.join(buffer))
                    buffer = []
                # Find the matching quote
                open_quote = True
                c = i.next()
                while c != u'"':
                    buffer.append(c)
                    c = i.next()
                if buffer:
                    word = u''.join(buffer).strip()
                    if word:
                        words.append(word)
                    buffer = []
                open_quote = False
            else:
                if not saw_loose_comma and c == u',':
                    saw_loose_comma = True
                buffer.append(c)
    except StopIteration:
        # If we were parsing an open quote which was never closed treat
        # the buffer as unquoted.
        if buffer:
            if open_quote and u',' in buffer:
                saw_loose_comma = True
            to_be_split.append(u''.join(buffer))
    if to_be_split:
        if saw_loose_comma:
            delimiter = u','
        else:
            delimiter = u' '
        for chunk in to_be_split:
            words.extend(split_strip(chunk, delimiter))
    #words = list(set(words))
    #words.sort()
    return words

def split_strip(input, delimiter=u','):
    """
    Splits ``input`` on ``delimiter``, stripping each resulting string
    and returning a list of non-empty strings.
    """
    if not input:
        return []

    words = [w.strip() for w in input.split(delimiter)]
    return [w for w in words if w]


#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""API module of universaltag via piston


Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'
from django import forms
from django.conf import settings
from django.http import Http404
from django.shortcuts import get_object_or_404
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.contenttypes.models import ContentType
from piston.handler import BaseHandler
from piston.utils import rc, validate, throttle

from ..models import TaggedItem
from ..utils import parse_tag_input
from ..exceptions import DuplicateError

class ValidationPOSTForm(forms.Form):
    """Form for POST validation"""
    labels = forms.CharField()
class ValidationPUTForm(forms.Form):
    """Form for PUT validation"""
    labels = forms.CharField(required=False)

def get_or_not_found(fn):
    """Get and set object or return rc.NOT_FOUND decorator
    
    Get object instance from content_type and object_id and set it to request.obj
    and call decorated function, or return rc.NOT_FOUND when object could not be found
    """
    def wrapper(self, request, content_type, object_id, *args, **kwargs):
        try:
            ctype = get_object_or_404(ContentType, pk=content_type)
            obj = ctype.get_object_for_this_type(pk=object_id)
            request.obj = obj
            return fn(self, request, content_type, object_id, *args, **kwargs)
        except (Http404, ObjectDoesNotExist):
            return rc.NOT_FOUND
    return wrapper

class TaggedItemHandler(BaseHandler):
    """TaggedItem piston handler"""
    allowed_method = ('GET', 'POST', 'PUT', 'DELETE')
    model = TaggedItem
    fields = (
        'pk',
        ('tag', ('pk', 'label', 'absolute_uri')),
        'frozen', 'order',
        'absolute_uri'
    )
    
    @get_or_not_found
    def read(self, request, content_type, object_id, label=None):
        qs = self.model.objects.get_for_object(request.obj)
        if label:
            return qs.get(tag__label=label)
        return qs
    
    @get_or_not_found
    @validate(ValidationPOSTForm, 'POST')
    @throttle(100, 10*60)
    def create(self, request, content_type, object_id):
        labels = parse_tag_input(request.form.cleaned_data['labels'])
        instance_list = []
        for label in labels:
            if len(label) > settings.UNIVERSALTAG_TAG_LENGTH:
                continue
            try:
                instance_list.append(self.model.objects.add_or_get(request.obj, label, False))
            except DuplicateError:
                pass
        return instance_list
    
    @get_or_not_found
    @validate(ValidationPUTForm, 'PUT')
    @throttle(100, 10*60)
    def update(self, request, content_type, object_id, label=None):
        if not label:
            # Sort tags via order of appearance in labels
            labels = parse_tag_input(request.form.cleaned_data['labels'])
            for i, label in enumerate(labels):
                tagged_item = self.model.objects.add_or_get(request.obj, label)
                tagged_item.order = i
                tagged_item.save()
            return TaggedItem.objects.get_for_object(request.obj)
        else:
            # Toggle tag freeze status
            # Only author of obj can freeze/thaw tag
            if hasattr(request, 'user') and request.user.is_authenticated():
                if request.user.is_superuser:
                    return self.model.objects.freeze(request.obj, label)
                for attr in settings.UNIVERSALTAG_AUTHOR_ATTRS:
                    if getattr(request.obj, attr, None) == request.user:
                        return self.model.objects.freeze(request.obj, label)
            return rc.FORBIDDEN
        
    @get_or_not_found
    @throttle(100, 10*60)
    def delete(self, request, content_type, object_id, label):
        if self.model.objects.remove(request.obj, label) is None:
            # Tried to delete frozen tag
            return rc.FORBIDDEN
        # Successfully deleted
        return rc.DELETED



#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""The short module explanation.

the long module explanation.
the long module explanation.

Methods:
    foobar - the explanation of the method.

Data:
    hogehoge - the explanation of the data.


Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'
from django.db import models
from django.contrib.auth.models import User

from universaltag.fields import UniversalTagField

class Entry(models.Model):
    title = models.CharField("title", max_length=128)
    body = models.TextField("body")
    author = models.ForeignKey(User, verbose_name="author")
    tags = UniversalTagField()
    
    def __unicode__(self):
        return self.title
    
    @models.permalink
    def get_absolute_url(self):
        return ('blogs-entry-detail', (self.pk,))


#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""Default modeltag settings


DATA:
    UNIVERSAL_TAG_LENGTH - the max string length of each tag
    UNIVERSAL_AUTHOR_ATTRS - list/tuple of attr name to get author of object
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'
from django.conf import settings

settings.UNIVERSALTAG_TAG_LENGTH = \
    getattr(settings, 'UNIVERSALTAG_TAG_LENGTH', 50)
settings.UNIVERSALTAG_AUTHOR_ATTRS = \
    getattr(settings, 'UNIVERSALTAG_AUTHOR_ATTRS', ('author', 'user', 'owner', 'created_by'))

from patches.piston_issue_140_monkey_patch import *



#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""Model module for universaltag

Classes:
    TagManager - a manager class of tag
    TaggedItemManager - a manager class of tagged item
    Tag - a model class of tag
    TaggedItem - a bridge model class of tag and model

Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'
from django.db import models
from django.db.models import Max
from django.conf import settings
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from exceptions import DuplicateError, NotDeletableError
from utils import parse_tag_input

class TagManager(models.Manager):
    """Manager class of tag model"""
        
    def get_for_model(self, model):
        """Return tags related to the model"""
        ctype = ContentType.objects.get_for_model(model)
        return self.filter(items__content_type=ctype).distinct()
    
    def get_for_object(self, obj):
        """Return tags related to the obj"""
        ctype = ContentType.objects.get_for_model(obj)
        return self.filter(items__content_type=ctype, items__object_id=obj.pk).distinct().order_by('items__order')
    
class TaggedItemManager(models.Manager):
    """Model manager for TaggedItem"""
    
    def get_by_label(self, obj, label):
        """Get tagged item by label"""
        qs = self.get_for_object(obj)
        return qs.get(tag__label=label)
    
    def get_for_model(self, model):
        """Return tagged_items related to the model"""
        ctype = ContentType.objects.get_for_model(model)
        return self.filter(content_type=ctype).distinct()
    
    def get_for_object(self, obj):
        """Return tagged_items related to the obj"""
        ctype = ContentType.objects.get_for_model(obj)
        return self.filter(content_type=ctype, object_id=obj.pk).distinct().order_by('order')

    def add_or_get(self, obj, label, ignore_duplication=True):
        """Add or get tag to obj and return TaggedItem instance
        
        Args:
            obj - the target model instance
            label - a label of tag
            ignore_duplication - if it False, DuplicateError occur when the label tag is already registered.
        
        Return:
            TaggedItem instance
        """
        ct = ContentType.objects.get_for_model(obj)
        tag = Tag.objects.get_or_create(label=label)[0]
        # Get or Create
        tagged_item, created = self.get_or_create(tag=tag, content_type=ct, object_id=obj.pk)
        if not created and not ignore_duplication:
            raise DuplicateError('%s is already related to the object' % label)
        return tagged_item
    
    def remove(self, obj, label, ignore_undeletable=True):
        """Remove label from the obj and return TaggedItem
        
        If the label is not related to any obj, the label will be removed as well.
        
        Args:
            obj - the target model instance
            label - a label of tag
        
        Return:
            TaggedItem instance - removed TaggedItem instance (instance.pk is None)
        """
        tagged_item = self.get_for_object(obj).get(tag__label=label)
        if tagged_item.frozen:
            if not ignore_undeletable:
                raise NotDeletableError("%s is a frozen tag for the object. couldn't remove." % label)
            else:
                return None
        else:
            tagged_item.delete()
            if tagged_item.tag.items.count() == 0:
                # Remove tag as well
                tagged_item.tag.delete()
        return tagged_item
            
    
    def freeze(self, obj, label, action=None):
        """Freeze or thaw label tag related to the obj
        
        Args:
            obj - the target model instance
            label - a label of tag
            action - None for toggle, `freeze` for freeze and `thaw` for thaw.
        
        Return:
            TaggedItem instance
        """
        tagged_item = self.get_for_object(obj).get(tag__label=label)
        if action == 'thaw' or (tagged_item.frozen and action is None):
            tagged_item.frozen = False
        elif action == 'freeze' or (not tagged_item.frozen and action is None):
            tagged_item.frozen = True
        tagged_item.save()
        return tagged_item
    
    def reconstruct(self, obj, labels):
        """Reconstruct tags for object via label list
        
        Args:
            obj - the target model instance
            labels - a list of tag label
        """
        ctype = ContentType.objects.get_for_model(obj)
        current_tags = list(Tag.objects.filter(items__content_type=ctype, items__object_id=obj.pk))
        updated_tag_labels = parse_tag_input(labels)
        # Remove all tags not in updated_tag_labels
        tags_for_removal = [tag for tag in current_tags if tag.label not in updated_tag_labels]
        if len(tags_for_removal):
            self.filter(
                content_type=ctype,
                object_id=obj.pk,
                tag__in=tags_for_removal).delete()
        # Append new tags
        current_tag_labels = [tag.label for tag in current_tags]
        for tag_label in updated_tag_labels:
            if not tag_label in current_tag_labels:
                self.add_or_get(obj, tag_label)
                
class Tag(models.Model):
    """Actual individual universaltag model"""
    label       = models.CharField(_('label'), 
                                   max_length=settings.UNIVERSALTAG_TAG_LENGTH, 
                                   unique=True, db_index=True)
    
    objects     = TagManager()
    
    class Meta:
        ordering            = ('label',)
        verbose_name        = _('tag')
        verbose_name_plural = _('tags')

    def __unicode__(self):
        return self.label
    
    @models.permalink
    def get_absolute_url(self):
        return ('universaltag-tag-detail', (self.label,))

class TaggedItem(models.Model):
    """Bridge model of tag and model with generic relation."""
    tag             = models.ForeignKey(Tag, verbose_name=_('tag'), related_name='items')
    content_type    = models.ForeignKey(ContentType, verbose_name=_('content type'), related_name="content_type_set_for_%(class)s")
    object_id       = models.PositiveIntegerField(_('object ID'))
    content_object  = generic.GenericForeignKey(ct_field="content_type", fk_field="object_id")
    
    # TODO: shoud be 'lock'
    frozen          = models.BooleanField(_('is not deletable'), default=False)
    # TODO: should use django's ordering system (May not usable, so have to find
    # out)
    order           = models.IntegerField(_('order'), default=-1, blank=True)
    
    objects         = TaggedItemManager()
    
    class Meta:
        ordering            = ('order',)
        unique_together     = ('tag', 'content_type', 'object_id')
        verbose_name        = _('tagged item')
        verbose_name_plural = _('tagged items')

    def __unicode__(self):
        return self.tag.__unicode__()

    def save(self, *args, **kwargs):
        # Automatically increment order
        if self.order == -1:
            queryset = TaggedItem.objects.get_for_object(self.content_object)
            if queryset.count() > 0:
                dict = queryset.aggregate(max=Max('order'))
                self.order = dict['max'] + 1
            else:
                self.order = 0
        super(TaggedItem, self).save(*args, **kwargs)
    
    def get_absolute_url(self):
        """Return permalink of object related."""
        if hasattr(self.content_object, 'get_absolute_url'):
            return self.content_object.get_absolute_url()
        return ""
    
    @models.permalink
    def get_api_url(self):
        return ("universaltag-api", (self.content_type.pk, self.object_id))



#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""Admin config for universaltag


Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'
from django.contrib import admin
from django.utils.text import ugettext_lazy as _

from models import Tag, TaggedItem

class TagAdmin(admin.ModelAdmin):
    """Admin configure for Tag model"""
    list_display    = ('__unicode__', 'number_of_items',)
    search_fields   = ('label',)
    
    def number_of_items(self, obj):
        return len(obj.items.iterators)
    number_of_items.short_description = _("Number of items")
    
class TaggedItemAdmin(admin.ModelAdmin):
    """Admin configure for TaggedItem model"""
    list_display    = ('__unicode__', 'content_object', 'content_type', 'frozen', 'order')
    list_filter     = ('content_type', 'frozen',)

admin.site.register(Tag, TagAdmin)
admin.site.register(TaggedItem, TaggedItemAdmin)



#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""Field module for universaltag

Classes:
    UniversalTagField - TaggedItem field for model
    

Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'
from django.contrib.contenttypes.generic import GenericRelation
from models import TaggedItem

class UniversalTagField(GenericRelation):
    """TaggedItem field for model (Reverse generic relation)
    
    Usage:
        >>> from django.db import models
        >>> class Book(models.Model):
        ...     title = models.CharField("title", max_length=50)
        ...     body = models.TextField("body")
        ...     tags = UniversalTagField()
        
    """
    def __init__(self, **kwargs):
        if not 'to' in kwargs:
            kwargs['to'] = TaggedItem
        super(UniversalTagField, self).__init__(**kwargs)

try:
    # Accept migration via south
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], [r'universaltag.fields.UniversalTagField'])
except ImportError:
    pass



#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""Template tags module for universaltag


Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'
from django import template
from django.conf import settings
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string
from django.template import TemplateSyntaxError
from django.contrib.contenttypes.models import ContentType

from ..models import Tag, TaggedItem

register = template.Library()
    
class RenderUniversalTagHeadNode(template.Node):
    def render(self, context):
        suggestions = Tag.objects.all()
        context.push()
        html = render_to_string('universaltag/head.html', {'suggestions': suggestions}, context)
        context.pop()
        return html
    
class RenderUniversalTagNode(template.Node):
    def __init__(self, object, threshold=None):
        self.object = template.Variable(object)
        self.threshold = template.Variable(threshold) if threshold else None
        
    def render(self, context):
        def is_freezable(request, object):
            if hasattr(request, 'user') and request.user.is_authenticated():
                if request.user.is_superuser:
                    return True
                for attr in settings.UNIVERSALTAG_AUTHOR_ATTRS:
                    if getattr(object, attr, None) == request.user:
                        return True
            return False
        request = template.resolve_variable('request', context)
        object = self.object.resolve(context)
        
        tagged_items = TaggedItem.objects.get_for_object(object)
        ctype = ContentType.objects.get_for_model(object)
        # Threshold
        if self.threshold:
            threshold = self.threshold.resolve(context)
            tagged_items = tagged_items[:threshold]
        # HTML
        context.push()
        kwargs = {
            'tagged_items': tagged_items,
            'universaltag_api_url': reverse('universaltag-api', args=(ctype.pk, object.pk)),
            'is_freezable': is_freezable(request, object),
        }
        html = render_to_string('universaltag/list.html', kwargs, context)
        context.pop()
        return html

class GetUniversalTagAPIURLNode(template.Node):
    def __init__(self, object, variable):
        self.object = template.Variable(object)
        self.variable = variable
    
    def render(self, context):
        object = self.object.resolve(context)
        ctype = ContentType.objects.get_for_model(object)
        context[self.variable] = reverse("universaltag-api", args=(ctype.pk, object.pk))
        return ""
                                  
@register.tag
def render_universaltag_tags(parser, token):
    """Render universaltag list as ul list
    
    Usage:
        {% render_universaltag_tags of <object> %}
        {% render_universaltag_tags of <object> threshold <threshold> %}
        
    """
    bits = token.split_contents()
    if len(bits) == 5:
        if bits[1] != 'of':
            raise TemplateSyntaxError("first argument of %s tag must be 'of'" % bits[0])
        elif bits[3] != 'threshold':
            raise TemplateSyntaxError("third argument of %s tag must be 'threshold'" % bits[0])
        return RenderUniversalTagNode(bits[2], bits[4])
    elif len(bits) == 3:
        if bits[1] != 'of':
            raise TemplateSyntaxError("first argument of %s tag must be 'of'" % bits[0])
        return RenderUniversalTagNode(bits[2])
    raise TemplateSyntaxError("%s tag takes exactly 3 or 5 arguments." % bits[0])

@register.tag
def render_universaltag_head(parser, token):
    """Render javascript and css to be able the feature of editing tags
    
    Use this template tag in head block to be able the feature of editing tags.
    
    Syntax:
        {% render_universaltag_head %}
    """
    bits = token.split_contents()
    if len(bits) == 1:
        return RenderUniversalTagHeadNode()
    raise TemplateSyntaxError("%s tag never takes any arguments." % bits[0])

@register.tag
def get_universaltag_api_url(parser, token):
    """Get universaltag api url
    
    Usage:
        {% get_universaltag_api_url of <object> to <variable> %}
        
    """
    bits = token.split_contents()
    if len(bits) == 5:
        if bits[1] != 'of':
            raise TemplateSyntaxError("first argument of %s tag must be 'of'" % bits[0])
        elif bits[3] != 'to':
            raise TemplateSyntaxError("third argument of %s tag must be 'to'" % bits[0])
        return GetUniversalTagAPIURLNode(bits[2], bits[4])
    raise TemplateSyntaxError("%s tag takes exactly 5 arguments." % bits[0])


#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""django_filters Filter class for universaltag

`django_filters` is a generic system for filtering Django QuerySets based on user selection.
See https://github.com/alex/django-filter for more detail.


Classes:
    ModeltagFilter - a custom filter for modeltag
    

Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'
from django import forms
from django.db.models import Count
from django.utils.translation import ugettext_lazy as _

from models import Tag
from utils import parse_tag_input

from django_filters import filters

    
class UniversalTagFilter(filters.ChoiceFilter):
    """
    This filter preforms an OR query on the selected options.
    """
    field_class = forms.ChoiceField
    def __init__(self, threshold=1, *args, **kwargs):
        """Constructor
        
        Args:
            threshold - threshold of number of tags. tags under it ignored to display
        """
        self.threshold = threshold
        super(UniversalTagFilter, self).__init__(*args, **kwargs)
        
    @property
    def field(self):
        qs = Tag.objects.get_for_model(self.model).annotate(count=Count('items')).exclude(count__lt=self.threshold).order_by('-count')
        choices = [['', _('All')]] + [(tag.pk, tag.label) for tag in qs]
        self.extra['choices'] = tuple(choices)
        return super(UniversalTagFilter, self).field
    
    def filter(self, qs, value):
        if not value:
            return qs
        if isinstance(value, basestring):
            value = parse_tag_input(value)
        elif not isinstance(value, list) and not isinstance(value, tuple):
            value = [value]
        return qs.filter(**{"%s__tag__pk__in"%self.name: value}).distinct()


#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""Test module for universaltag models


Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'
from django.contrib.auth.models import User
from app_test import AppTestCase
from app.models import Book
from ..models import TaggedItem

class TaggedItemTestCase(AppTestCase):
    installed_apps = ['universaltag.tests.app']
    def setUp(self):
        self.admin = User.objects.create_superuser(
                username='universaltag_test_user', email='universaltag_test_user@test.com',
                password='password'
            )
        self.book1 = Book(pk=1, title="foo", body="foofoofoo", author=self.admin)
        self.book1.save()
        self.book2 = Book(pk=2, title="bar", body="barbarbar", author=self.admin)
        self.book2.save()
    
        # Delete TaggedItem everytime
        TaggedItem.objects.all().delete()
        
    def test_add_or_get(self):
        TaggedItem.objects.add_or_get(self.book1, 'test1')
        TaggedItem.objects.add_or_get(self.book1, 'test2')
        TaggedItem.objects.add_or_get(self.book1, 'test3')
        self.assertEquals(len(self.book1.tags.values()), 3)
    def test_add_or_get_duplicate(self):
        TaggedItem.objects.add_or_get(self.book1, 'test1')
        TaggedItem.objects.add_or_get(self.book1, 'test2')
        TaggedItem.objects.add_or_get(self.book1, 'test3')
        # Add duplicate tag
        TaggedItem.objects.add_or_get(self.book1, 'test3')
        self.assertEquals(len(self.book1.tags.values()), 3)
    def test_get_for_model(self):
        TaggedItem.objects.add_or_get(self.book1, 'test1')
        TaggedItem.objects.add_or_get(self.book1, 'test2')
        TaggedItem.objects.add_or_get(self.book1, 'test3')
        TaggedItem.objects.add_or_get(self.book2, 'test2')
        TaggedItem.objects.add_or_get(self.book2, 'test3')
        TaggedItem.objects.add_or_get(self.book2, 'test4')

        qs = TaggedItem.objects.get_for_model(Book)
        self.assertEquals(qs.count(), 6)
    def test_get_for_object(self):
        TaggedItem.objects.add_or_get(self.book1, 'test1')
        TaggedItem.objects.add_or_get(self.book1, 'test2')
        TaggedItem.objects.add_or_get(self.book1, 'test3')
        TaggedItem.objects.add_or_get(self.book2, 'test2')
        TaggedItem.objects.add_or_get(self.book2, 'test3')
        TaggedItem.objects.add_or_get(self.book2, 'test4')

        qs = TaggedItem.objects.get_for_object(self.book1)
        self.assertEquals(qs.count(), 3)
    def test_remove(self):
        TaggedItem.objects.add_or_get(self.book1, 'test1')
        TaggedItem.objects.add_or_get(self.book1, 'test2')
        TaggedItem.objects.add_or_get(self.book1, 'test3')

        qs = TaggedItem.objects.get_for_object(self.book1)
        self.assertEquals(qs.count(), 3)
        
        TaggedItem.objects.remove(self.book1, 'test1')
        
        qs = TaggedItem.objects.get_for_object(self.book1)
        self.assertEquals(qs.count(), 2)
        
    def test_freeze(self):
        TaggedItem.objects.add_or_get(self.book1, 'test1')
        TaggedItem.objects.add_or_get(self.book1, 'test2')
        TaggedItem.objects.add_or_get(self.book1, 'test3')
        
        TaggedItem.objects.freeze(self.book1, 'test1', 'thaw')
        
        tagged_item = TaggedItem.objects.get_for_object(self.book1).get(tag__label='test1')
        self.assertFalse(tagged_item.frozen)

        TaggedItem.objects.freeze(self.book1, 'test1')
        
        tagged_item = TaggedItem.objects.get_for_object(self.book1).get(tag__label='test1')
        self.assertTrue(tagged_item.frozen)
        
        TaggedItem.objects.freeze(self.book1, 'test1')
        
        tagged_item = TaggedItem.objects.get_for_object(self.book1).get(tag__label='test1')
        self.assertFalse(tagged_item.frozen)
    
    def test_remove_fail(self):
        TaggedItem.objects.add_or_get(self.book1, 'test1')
        TaggedItem.objects.add_or_get(self.book1, 'test2')
        TaggedItem.objects.add_or_get(self.book1, 'test3')
        
        TaggedItem.objects.freeze(self.book1, 'test1', 'freeze')
        TaggedItem.objects.remove(self.book1, 'test1')
        
        qs = TaggedItem.objects.get_for_object(self.book1)
        self.assertEquals(qs.count(), 3)
        
        TaggedItem.objects.freeze(self.book1, 'test1', 'thaw')
        TaggedItem.objects.remove(self.book1, 'test1')
        
        qs = TaggedItem.objects.get_for_object(self.book1)
        self.assertEquals(qs.count(), 2)
    
    def test_reconstruct(self):
        TaggedItem.objects.add_or_get(self.book1, 'test1')
        TaggedItem.objects.add_or_get(self.book1, 'test2')
        TaggedItem.objects.add_or_get(self.book1, 'test3')
        
        NEW_TAGS = ("foobar1", "foobar2", "foobar3", "foobar4")
        TaggedItem.objects.reconstruct(self.book1, ",".join(NEW_TAGS))
        
        qs = TaggedItem.objects.get_for_object(self.book1)
        for i, tagged_item in enumerate(qs):
            self.assertEquals(tagged_item.tag.label, NEW_TAGS[i])


        



from django.conf.urls.defaults import patterns, include, url

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    url(r'^admin/doc/',     include('django.contrib.admindocs.urls')),
    url(r'^admin/',         include(admin.site.urls)),
    url(r'^',               include('miniblog.blogs.urls')),
    url(r'^tags/',          include('universaltag.urls')),        
)



#
# Monkey Patch for fixing django-piston issue#140: 'GenericForeignKey' object has no attribute 'serialize'
# Load this module on app/project '__init__.py' as the following code to fix the issue.
#
#   from piston_issue_140_monkey_patch import *
#
# See comment#9 of the issue 
# https://bitbucket.org/jespern/django-piston/issue/140/genericforeignkey-object-has-no-attribute#comment-357757
from django.contrib.contenttypes.generic import GenericForeignKey
GenericForeignKey.serialize = None



#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""Exceptions for universaltag


Exceptions:
    DuplicateError - This exception occur when user tried to add duplicate tag
    NotDeletableError - This exception occur when user tried to delete frozen-tag

Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'

class DuplicateError(Exception):
    """This exception occur when user tried to add duplicate tag"""
    pass

class NotDeletableError(Exception):
    """This exception occur when user tried to delete frozen-tag"""
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# author:        alisue
# date:            2011/03/22
#
from setuptools import setup, find_packages

version = "0.1.5"

def read(filename):
    import os.path
    return open(os.path.join(os.path.dirname(__file__), filename)).read()

setup(
    name="django-universaltag",
    version=version,
    description = "Universal tagging library for Django",
    long_description=read('README.rst'),
    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
    keywords = "django tagging universal tag",
    author = "Alisue",
    author_email = "lambdalisue@hashnote.net",
    url=r"https://github.com/lambdalisue/django-universaltag",
    download_url = r"https://github.com/lambdalisue/django-universaltag/tarball/master",
    license = 'MIT',
    packages = find_packages(exclude=['tests']),
    include_package_data = True,
    zip_safe=False,
    install_requires=[
        'distribute',
        'setuptools-git',
        'django>=1.2',
        'django-piston>=0.2.2',
    ],
    tests_require = [
    ],
    test_suite = 'runtests.runtests',
)




#!/usr/bin/env python
# vim: set fileencoding=utf8 :
"""The short module explanation.

the long module explanation.
the long module explanation.

Methods:
    foobar - the explanation of the method.

Data:
    hogehoge - the explanation of the data.


Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__author__  = 'Alisue <lambdalisue@hashnote.net>'
__version__ = '1.0.0'
__date__    = '2011/06/09'
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from app_test import AppTestCase
from app.models import Book
from ..models import Tag, TaggedItem

import os.path

class GeneralViewTestCase(AppTestCase):
    installed_apps = ['universaltag.tests.app']
    urls = 'universaltag.urls'
    fixtures = ['universaltag_test_data.json',]
    template_dirs = (
        os.path.join(os.path.dirname(__file__), 'templates'),
    )
    
    def setUp(self):
        # Set Test TEMPLATE_DIRS
        self._template_dirs_backup = settings.TEMPLATE_DIRS
        settings.TEMPLATE_DIRS = self.template_dirs
        # Update content_type and book
        self.book = Book.objects.get(pk=1)
        self.book_pk = self.book.pk
        self.book_ct = ContentType.objects.get_for_model(Book).pk
        
    def tearDown(self):
        settings.TEMPLATE_DIRS = self._template_dirs_backup
        
    def test_list(self):
        response = self.client.get('/')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.context['object_list'].count(), Tag.objects.count())
    
    def test_detail(self):
        response = self.client.get('/tag1/')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.context['object'], Tag.objects.get(label='tag1'))
    
    def test_api_get(self):
        url = "/api/%d/%d/" % (self.book_ct, self.book_pk)
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
    
    def test_api_post(self):
        url = "/api/%d/%d/" % (self.book_ct, self.book_pk)
        data = {
            'labels': "test1, test2, test3",
        }
        previous_count = Tag.objects.count()
        response = self.client.post(url, data)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(Tag.objects.count(), previous_count+3)
    
    def test_api_put_sort(self):
        url = "/api/%d/%d/" % (self.book_ct, self.book_pk)
        # Reverse
        labels = ['tag3', 'tag2', 'tag1']
        data = {
            'labels': ", ".join(labels),
        }
        response = self.client.put(url, data)
        self.assertEquals(response.status_code, 200)
        for i, tagged_item in enumerate(TaggedItem.objects.get_for_object(self.book)):
            self.assertEquals(tagged_item.tag.label, labels[i])
        # Re reverse
        labels.reverse()
        data = {
            'labels': ", ".join(labels),
        }
        response = self.client.put(url, data)
        self.assertEquals(response.status_code, 200)
        for i, tagged_item in enumerate(TaggedItem.objects.get_for_object(self.book)):
            self.assertEquals(tagged_item.tag.label, labels[i])
    
    def test_api_put_freeze(self):
        url = "/api/%d/%d/tag1/" % (self.book_ct, self.book_pk)
        
        # Thaw programatically
        tagged_item = TaggedItem.objects.get_by_label(self.book, 'tag1')
        tagged_item.frozen = False
        tagged_item.save()
        
        # Failed for anonymous user
        response = self.client.put(url)
        self.assertEquals(response.status_code, 401)
        
        # Required to be authenticated user
        self.assertTrue(self.client.login(username='admin', password='password'))
        
        # Freeze
        response = self.client.put(url)
        self.assertEquals(response.status_code, 200)
        self.assertTrue(TaggedItem.objects.get_by_label(self.book, 'tag1').frozen)
        
        # Thaw
        response = self.client.put(url)
        self.assertEquals(response.status_code, 200)
        self.assertFalse(TaggedItem.objects.get_by_label(self.book, 'tag1').frozen)
    
    def test_api_delete(self):
        # Create New Tag for delete
        TaggedItem.objects.add_or_get(self.book, "delete_test")
        # Delete
        url = "/api/%d/%d/delete_test/" % (self.book_ct, self.book_pk)
        previous_count = TaggedItem.objects.get_for_object(self.book).count()
        
        response = self.client.delete(url)
        self.assertEquals(response.status_code, 204)
        self.assertEquals(TaggedItem.objects.get_for_object(self.book).count(), previous_count-1)
